=begin
# Author:         tomot
# Name :          Wall-Section tool 1
# Description :   Create a 2D wall section 
# Usage :         click 2 points; draws a 2D wall section 
# Date :          2011,07,02
# Type :          Tool
# Revised:
#-----------------------------------------------------------------------------
=end
require 'sketchup.rb'

class WallSectionTool1

# This is the standard Ruby initialize method that is called when you create
# a new object.
def initialize
  Sketchup.send_action(CMD_VIEW_FRONT)
  
  @ip1 = nil
  @ip2 = nil
  @xdown = 0
  @zdown = 0
  
  # sets the default wall settings
  @ext = 0.5.inch if not @ext
  @width = 6.inch if not @width
  @int = 0.5.inch if not @int
    
  # Dialog box
  ins = ["YES","NO"]
  enums = ["", "", "", ins.join("|")]
  prompts = ["Ext. Fin.", "Wall Thickness ","Int. Fin." ,"Insulation"]
  values = [@ext, @width, @int, @insul]
  results = inputbox prompts, values, enums, "Wall Section parameters"
  
  return nil if not results
     @ext, @width, @int, @insul = results
          
end

# The activate method is called by SketchUp when the tool is first selected.
# it is a good place to put most of your initialization
def activate

    # The Sketchup::InputPoint class is used to get 3D points from screen
    # positions.  It uses the SketchUp inferencing code.
    # In this tool, we will have two points for the endpoints of the line.
    @ip1 = Sketchup::InputPoint.new
    @ip2 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false
    
    # This sets the label for the VCB
    Sketchup::set_status_text " Pick Wall Start", SB_VCB_LABEL
    
    self.reset(nil)

end

# deactivate is called when the tool is deactivated because
# a different tool was selected
def deactivate(view)
    view.invalidate if @drawn
end

# The onMouseMove method is called whenever the user moves the mouse.
# because it is called so often, it is important to try to make it efficient.
# In a lot of tools, your main interaction will occur in this method.
def onMouseMove(flags, x, z, view)
    if( @state == 0 )
        # We are getting the first end of the line.  Call the pick method
        # on the InputPoint to get a 3D position from the 2D screen position
        # that is bassed as an argument to this method.
        @ip.pick view, x, z
        if( @ip != @ip1 )
            # if the point has changed from the last one we got, then
            # see if we need to display the point.  We need to display it
            # if it has a display representation or if the previous point
            # was displayed.  The invalidate method on the view is used
            # to tell the view that something has changed so that you need
            # to refresh the view.
            view.invalidate if( @ip.display? or @ip1.display? )
            @ip1.copy! @ip
            
            # set the tooltip that should be displayed to this point
            view.tooltip = @ip1.tooltip
        end
    else
        # Getting the second end of the line
        # If you pass in another InputPoint on the pick method of InputPoint
        # it uses that second point to do additional inferencing such as
        # parallel to an axis.
        @ip2.pick view, x, z, @ip1
        view.tooltip = @ip2.tooltip if( @ip2.valid? )
        view.invalidate
        
        # Update the length displayed in the VCB
        if( @ip2.valid? )
            length = @ip1.position.distance(@ip2.position)
            Sketchup::set_status_text length.to_s, SB_VCB_VALUE
        end
        
        # Check to see if the mouse was moved far enough to create a line.
        # This is used so that you can create a line by either draggin
        # or doing click-move-click
        if( (x-@xdown).abs > 10 || (z-@zdown).abs > 10 )
            @dragging = true
        end
    end
end

# The onLButtonDOwn method is called when the user presses the left mouse button.
def onLButtonDown(flags, x, z, view)
    # When the user clicks the first time, we switch to getting the
    # second point.  When they click a second time we create the line
    if( @state == 0 )
        @ip1.pick view, x, z
        if( @ip1.valid? )
            @state = 1
            Sketchup::set_status_text "Pick Wall Height", SB_PROMPT
            @xdown = x
            @ydown = z
        end
    else
        # create the line on the second click
        if( @ip2.valid? )
            self.create_geometry(@ip1.position, @ip2.position,view)
            self.reset(view)
        end
    end
    
    # Clear any inference lock
    view.lock_inference
end

# The onLButtonUp method is called when the user releases the left mouse button.
def onLButtonUp(flags, x, z, view)
    # If we are doing a drag, then create the line on the mouse up event
    if( @dragging && @ip2.valid? )
        self.create_geometry(@ip1.position, @ip2.position,view)
        self.reset(view)
    end
end

# onKeyDown is called when the user presses a key on the keyboard.
# We are checking it here to see if the user pressed the shift key
# so that we can do inference locking
def onKeyDown(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
        @shift_down_time = Time.now
        
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
            # calling lock_inference with no arguments actually unlocks
            view.lock_inference
        elsif( @state == 0 && @ip1.valid? )
            view.lock_inference @ip1
        elsif( @state == 1 && @ip2.valid? )
            view.lock_inference @ip2, @ip1
        end
    end
end

# onKeyUp is called when the user releases the key
# We use this to unlock the interence
# If the user holds down the shift key for more than 1/2 second, then we
# unlock the inference on the release.  Otherwise, the user presses shift
# once to lock and a second time to unlock.
def onKeyUp(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end

# onUserText is called when the user enters something into the VCB
# In this implementation, we create a line of the entered length if
# the user types a length while selecting the second point
def onUserText(text, view)
    # We only accept input when the state is 1 (i.e. getting the second point)
    # This could be enhanced to also modify the last line created if a length
    # is entered after creating a line.
    return if not @state == 1
    return if not @ip2.valid?
    
    # The user may type in something that we can't parse as a length
    # so we set up some exception handling to trap that
    begin
        value = text.to_l
    rescue
        # Error parsing the text
        UI.beep
        puts "Cannot convert #{text} to a Length"
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value

    # Compute the direction and the second point
    pt1 = @ip1.position
    vec = @ip2.position - pt1
    if( vec.length == 0.0 )
        UI.beep
        return
    end
    vec.length = value
    pt2 = pt1 + vec

    # Create a line
    self.create_geometry(pt1, pt2, view)
    self.reset(view)
end

# The draw method is called whenever the view is refreshed.  It lets the
# tool draw any temporary geometry that it needs to.
def draw(view)
    if( @ip1.valid? )
        if( @ip1.display? )
            @ip1.draw(view)
            @drawn = true
        end
        
        if( @ip2.valid? )
            @ip2.draw(view) if( @ip2.display? )
            
            # The set_color_from_line method determines what color
            # to use to draw a line based on its direction.  For example
            # red, green or blue.
            view.set_color_from_line(@ip1, @ip2)
            self.draw_geometry(@ip1.position, @ip2.position, view)
            @drawn = true
        end
    end
end

# onCancel is called when the user hits the escape key
def onCancel(flag, view)
    self.reset(view)
end


# The following methods are not directly called from SketchUp.  They are
# internal methods that are used to support the other methods in this class.

# Reset the tool back to its initial state
def reset(view)
    # This variable keeps track of which point we are currently getting
    @state = 0
    
    # Display a prompt on the status bar
    #Sketchup::set_status_text("Click anywhere to start", SB_PROMPT)
    
    # clear the InputPoints
    @ip1.clear
    @ip2.clear
    
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
    
    @drawn = false
    @dragging = false
end

# Create new geometry when the user has selected two points.
def create_geometry(p1, p2, view)
    
    # define a new group
    model = Sketchup.active_model
    group = Sketchup.active_model.entities.add_group
    entities = group.entities
    
    model.start_operation "Create Wall Section"
    entities=model.active_entities
    
    # creates a new point from p1 in the direction of p2-p3 with length d
    # params are Point3d, 2 vertices, a length, returns a Point3d
    def translate(p1, p2, p3, d)
        v = p3 - p2
        v.length = d
        trans=Geom::Transformation.translation(v)
        return p1.transform(trans)
    end 
 
    if( @axis == "Yes" )
      view.model.entities.add_cline(p1,p2)
    end  
    
    pt1 = @ip1.position 
    pt2 = @ip2.position 
    pt2 = p2
        
    #----pi is here only for testing purposes
    pi = 3.141592653589793   # 180 degree angle = pi radians
                
    @pt1=Geom::Point3d.new(pt1[0],pt1[1],pt1[2])
    @pt2=Geom::Point3d.new(pt2[0],pt2[1],pt2[2])
    #---- distance btwn. input points @pt1 & @pt2 (height of wallsection)
        
    @vec=@pt2.distance @pt1
    
    @vector1=@pt1.vector_to( @pt2 )
    @vx=@vector1.x
    @vz=@vector1.z
    @rad=Math::tan @vz/@vx     
    
    
    
    #@angle=@pt2.angle_between @pt1

    @pt3=translate(@pt1, @pt1, @pt2, 1.5)
    @pt4=translate(@pt1, @pt1, @pt2, @vec-1.5)
    @pt5=translate(@pt1, @pt1, @pt2, @vec-3.0)
    @pt6=translate(@pt1, @pt1, @pt2, 3.5)
    @pt7=translate(@pt1, @pt1, @pt2, 2.5)
    
    #---- distance btwn. input points @pt1 & @pt3 (wall plate thickness/2)
    @vec2=(@pt3.distance @pt1)/2
   
    #------Compute new point pt11, @width (6") and at the same angle plus 90 degrees from pt1
    vec = @ip2.position - @pt1
    ang = (Math::PI / 2.0)
    vec.length = @width # @width of wall (6")
    translate = Geom::Transformation.translation(vec)
    rotate = Geom::Transformation.rotation( @pt1, Geom::Vector3d.new(0, 1, 0), ang)
    trans = @pt1.transform(translate)
    @pt11 = trans.transform(rotate)
       
    #------Compute new point pt22, @width (6") and at the same angle plus 90 degrees from pt2
    vec = @ip1.position - @pt2
    ang = (Math::PI / 2.0)
    vec.length = @width # @width of wall
    translate = Geom::Transformation.translation(vec)
    rotate = Geom::Transformation.rotation( @pt2, Geom::Vector3d.new(0, 1, 0), (0-ang))
    trans = @pt2.transform(translate)
    @pt22 = trans.transform(rotate)
    #...compute points req'd for wall        
    @pt33=translate(@pt11, @pt11, @pt22, 1.5)
    @pt44=translate(@pt11, @pt11, @pt22, @vec-1.5)
    @pt55=translate(@pt11, @pt11, @pt22, @vec-3.0)
    @pt66=translate(@pt11, @pt11, @pt22, 3.5)
    @pt77=translate(@pt11, @pt11, @pt22, 2.5)
    
    
    @pt02=translate(@pt22, @pt22, @pt2, @width+0.5)
    @pt01=translate(@pt11, @pt11, @pt1, @width+0.5)
    @pt002=translate(@pt2, @pt2, @pt22, @width+0.5)
    @pt001=translate(@pt1, @pt1, @pt11, @width+0.5)
    #...compute points req'd for insulation arc #1 thru #3
    @pt330=translate(@pt3, @pt3, @pt33, 1.0)
    @pt660=translate(@pt6, @pt6, @pt66, 1.0)
    @pt770=translate(@pt7, @pt7, @pt77, 1.0)
    @pt3300=translate(@pt3, @pt3, @pt33, @width-1.0) 
    @pt6600=translate(@pt6, @pt6, @pt66, @width-1.0) 
    @pt7700=translate(@pt7, @pt7, @pt77, @width-1.0) 
       
    #base=entities.add_face(@pt1, @pt2, @pt22, @pt11)  
    
    group=entities.add_group
    entities=group.entities
    
    #...do btm plate
    base=entities.add_face(@pt1, @pt3, @pt33, @pt11) 
    base=entities.add_line(@pt1, @pt33)
    base=entities.add_line(@pt3, @pt11)
    #...do dbl plate
    base=entities.add_face(@pt2, @pt22, @pt44, @pt4) 
    base=entities.add_line(@pt2, @pt44)
    base=entities.add_line(@pt4, @pt22)
    #...do top plate
    base=entities.add_face(@pt4, @pt44, @pt55, @pt5) 
    base=entities.add_line(@pt5, @pt44)
    base=entities.add_line(@pt4, @pt55)
    #...do wall between
    base=entities.add_line(@pt3, @pt5)
    base=entities.add_line(@pt33, @pt55)
    #...do ext sheathing
    base=entities.add_line(@pt01, @pt02)   
    #...do int finish
    base=entities.add_line(@pt001, @pt002)


if( @insul != "NO" ) 

    #...distance btwn. plates for insulation
    @vec1=@pt3.distance @pt5
    @vec3=@pt6.distance @pt3
    #...get the number of insul batts required for the wall height @vec1/@vec3
    @num_insul=(@vec1/@vec3).to_i 
    
    definitions=model.definitions
    count=definitions.add
    entities=count.entities
    
    # -------Draw first Insul Batt
    #--------1.draw the 180 degree insulation arc
    center = Geom::Point3d.new @pt770
    normal = Geom::Vector3d.new 0,1,0 
    xaxis = Geom::Vector3d.new 0,0,1  # rotated 90 quadrant
    start_a = 0.0
    end_a = Math::PI
    edgearray = entities.add_arc center, xaxis, normal, -1, start_a, end_a
    edge = edgearray[0]
    arccurve = edge.curve
    end_angle = arccurve.end_angle
    # -------2.draw the 90 degree insulation arc
    center = Geom::Point3d.new @pt6600 # new center location 90 degree arc
    normal = Geom::Vector3d.new 0,1,0 
    xaxis = Geom::Vector3d.new 1,0,0   # rotated quadrant
    start_a = 0.0
    end_a = Math::PI/2
    edgearray = entities.add_arc center, xaxis, normal, 1, start_a, end_a
    edge = edgearray[0]
    arccurve = edge.curve
    end_angle = arccurve.end_angle
    # -------3.draw the 90 degree insulation arc
    center = Geom::Point3d.new @pt3300 # new center location 90 degree arc
    normal = Geom::Vector3d.new 0,1,0  
    xaxis = Geom::Vector3d.new 0,0,1   # rotated quadrant
    start_a = 0.0
    end_a = Math::PI/2
    edgearray = entities.add_arc center, xaxis, normal, 1, start_a, end_a
    edge = edgearray[0]
    arccurve = edge.curve
    end_angle = arccurve.end_angle
    base=entities.add_line(@pt330, @pt7700)
    base=entities.add_line(@pt660, @pt7700)
    
    #------Do transform of location  
    t=Geom::Transformation.translation(Geom::Vector3d.new(0, 0, 0))
    entities=model.active_entities
    
    entities.add_instance(count, t)
    
    #------Copy Insul Batts to their new locations
    i = 1
    while i < @num_insul
       # Transformation
       i = i + 1
       vec = @pt5 - @pt6
       vec.length = @vec3*(i-1)
       t=Geom::Transformation.translation(Geom::Vector3d.new(0,@vec3*(i-1), 0))
       t=Geom::Transformation.translation(vec)
       entities.add_instance(count, t)
    end  
end # if insul    
    
    #Sketchup.send_action(CMD_ZOOM_EXTENTS)
    model.commit_operation
    
end

# Draw the geometry
def draw_geometry(pt1, pt2, view)
    view.draw_line(pt1, pt2)
end

end # class WallSectionTool1

#-----------------------------------------------------------------------------
def wallsectiontool1
    Sketchup.active_model.select_tool WallSectionTool1.new
end

if( not file_loaded?("WallSectionTool1.rb") )
    add_separator_to_menu("Plugins")
    UI.menu("Plugins").add_item("Wall Section Tool 1") { Sketchup.active_model.select_tool WallSectionTool1.new }
end
#-----------------------------------------------------------------------------
file_loaded("WallSectionTool1.rb")
